import React, { useState, useRef, useEffect } from 'react';
import classNames from 'classnames';
import createDimension, { useDimension } from 'dymension';
import Toast from 'react-bootstrap/Toast';

function _objectWithoutPropertiesLoose(source, excluded) {
  if (source == null) return {};
  var target = {};
  var sourceKeys = Object.keys(source);
  var key, i;

  for (i = 0; i < sourceKeys.length; i++) {
    key = sourceKeys[i];
    if (excluded.indexOf(key) >= 0) continue;
    target[key] = source[key];
  }

  return target;
}

var _excluded = ["header", "body", "bg", "className", "classEnter", "classExit", "duration", "dismissible", "onMount", "onUnmount", "onDismissed"];
var Toaster = function Toaster(_ref) {
  var _classNames, _classNames2;

  var header = _ref.header,
      body = _ref.body,
      bg = _ref.bg,
      className = _ref.className,
      classEnter = _ref.classEnter,
      classExit = _ref.classExit,
      duration = _ref.duration,
      dismissible = _ref.dismissible,
      onMount = _ref.onMount,
      onUnmount = _ref.onUnmount,
      onDismissed = _ref.onDismissed,
      props = _objectWithoutPropertiesLoose(_ref, _excluded);

  var _useDimension = useDimension(),
      show = _useDimension.show,
      resolve = _useDimension.resolve;

  var _useState = useState(false),
      pause = _useState[0],
      setPause = _useState[1];

  var observables = useRef({
    start: Date.now(),
    duration: duration
  }).current;
  useEffect(function () {
    observables.start = Date.now();
    observables.duration = duration;
  }, [duration]); //@ts-ignore

  useEffect(function () {
    if (pause) {
      observables.duration = Math.max(0, observables.duration - (Date.now() - observables.start));
    } else if (observables.duration > 0 && show) {
      var cleanup = setTimeout(function () {
        observables.duration = 0;
        resolve(true);
      }, observables.duration);
      return function () {
        return clearTimeout(cleanup);
      };
    } // eslint-disable-next-line react-hooks/exhaustive-deps

  }, [duration, show, pause]);
  useEffect(function () {
    onMount && onMount();
    return function () {
      return onUnmount && onUnmount();
    }; // eslint-disable-next-line react-hooks/exhaustive-deps
  }, [onMount, onUnmount]);

  var handleClose = function handleClose() {
    observables.duration = 0;
    resolve(true);
    onDismissed && onDismissed();
  };

  var handleMouseEnter = function handleMouseEnter() {
    if (observables.duration > 0) setPause(true);
  };

  var handleMouseLeave = function handleMouseLeave() {
    if (observables.duration > 0) {
      setPause(false);
      observables.start = Date.now();
    }
  };

  return React.createElement(Toast, Object.assign({
    className: classNames(className, (_classNames = {}, _classNames[classEnter] = show, _classNames[classExit] = !show, _classNames)),
    onClose: handleClose,
    onMouseEnter: handleMouseEnter,
    onMouseLeave: handleMouseLeave,
    bg: bg
  }, props), React.createElement(Toast.Header, {
    closeButton: dismissible
  }, header, duration > 0 && React.createElement("div", {
    className: classNames("toast-progress", (_classNames2 = {}, _classNames2["text-" + bg] = bg !== 'light', _classNames2))
  }, React.createElement("div", {
    className: classNames("toast-progress-bar", {
      pause: pause
    }),
    style: {
      animationDuration: duration + "ms"
    }
  }))), React.createElement(Toast.Body, {
    className: classNames("", {
      "text-white": bg === "dark"
    })
  }, body));
};
Toaster.defaultProps = {
  dismissible: false,
  duration: 6000,
  classEnter: 'rb-fade-toast-in',
  classExit: 'rb-fade-toast-out',
  bg: 'light'
};

var createNode = function createNode(parent) {
  if (typeof document !== "undefined") {
    return (parent || document.body).appendChild(document.createElement('div'));
  }
};

var toastContainer = /*#__PURE__*/createNode();

var setNodePosition = function setNodePosition(position) {
  var node = createNode(toastContainer);

  if (node) {
    node.id = "rb-toast-" + position;
  }

  return node;
};

var container = /*#__PURE__*/['top', 'bottom'].reduce(function (result, primePosition) {
  if (['top', 'bottom'].includes(primePosition)) {
    result[primePosition] = setNodePosition(primePosition);
  }

  return ['left', 'right'].reduce(function (result, position) {
    var nodeProsition = primePosition + "-" + position;
    result[nodeProsition] = setNodePosition(nodeProsition);
    return result;
  }, result);
}, {});
var isToastKeyExist = function isToastKeyExist(key, placement) {
  if (typeof document !== "undefined") {
    var _document$getElementB;

    var existNode = document.getElementById(key);
    return !!existNode === true && (placement ? ((_document$getElementB = document.getElementById(placement)) == null ? void 0 : _document$getElementB.contains(existNode)) === true : true);
  }

  return false;
};
var index = /*#__PURE__*/createDimension(Toaster, {
  delay: function delay(_ref2) {
    var _ref2$delay = _ref2.delay,
        _delay = _ref2$delay === void 0 ? 600 : _ref2$delay;

    return _delay;
  },
  containerNode: function containerNode(_ref3) {
    var _ref3$placement = _ref3.placement,
        placement = _ref3$placement === void 0 ? 'top-right' : _ref3$placement;
    return container[placement];
  },
  manipulateWrapperNode: function manipulateWrapperNode(wrapperNode, props) {
    if (props.key && typeof document !== "undefined") {
      var existNode = document.getElementById(props.key);

      if (existNode) {
        wrapperNode = existNode;
      } else {
        wrapperNode.id = props.key;
      }
    }

    wrapperNode.className = 'rb-toast-wrapper';
    return wrapperNode;
  }
});

export default index;
export { Toaster, isToastKeyExist };
//# sourceMappingURL=react-bootstrap-toast.esm.js.map
